package com.test.util;

import com.test.entity.SystemData;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import oshi.SystemInfo;
import oshi.hardware.HWDiskStore;
import oshi.hardware.HardwareAbstractionLayer;
import oshi.hardware.NetworkIF;
import oshi.software.os.OperatingSystem;

import java.io.IOException;
import java.net.NetworkInterface;
import java.util.Properties;

@Slf4j
@Service
public class SystemDataGetter implements DataGetter<SystemData>{

    @Resource
    SystemInfo info;

    private final Properties properties = System.getProperties();

    public SystemData getData() {
        OperatingSystem os = info.getOperatingSystem();
        HardwareAbstractionLayer hardware = info.getHardware();
        double memory = hardware.getMemory().getTotal() / 1024.0 / 1024 / 1024;
        double diskSize = hardware.getDiskStores().stream().mapToLong(HWDiskStore::getSize).sum()  / 1024.0 / 1024 / 1024;
        try {
            String ip = null;
            for (NetworkIF network : hardware.getNetworkIFs()) {
                String[] iPv4addr = network.getIPv4addr();
                NetworkInterface ni = network.queryNetworkInterface();
                if(!ni.isLoopback() && !ni.isPointToPoint() && ni.isUp() && !ni.isVirtual()
                        && (ni.getName().startsWith("eth") || ni.getName().startsWith("en"))
                        && iPv4addr.length > 0) {
                    ip = network.getIPv4addr()[0];
                    break;
                }
            }
            return new SystemData()
                    .setOsBit(os.getBitness())
                    .setOsArch(properties.getProperty("os.arch"))
                    .setOsVersion(os.getVersionInfo().getVersion())
                    .setOsName(os.getFamily())
                    .setCpuCore(hardware.getProcessor().getLogicalProcessorCount())
                    .setMemory(memory)
                    .setDisk(diskSize)
                    .setIp(ip);
        } catch (IOException e) {
            log.error("读取系统网络配置时出错", e);
            return null;
        }
    }
}
